package com.xs.core.common.validation;

import cn.hutool.core.text.CharSequenceUtil;
import com.xs.core.common.exception.BusinessException;
import org.springframework.context.MessageSource;
import org.springframework.context.i18n.LocaleContextHolder;

import java.util.Locale;
import java.util.function.BooleanSupplier;

public class CheckUtils extends Validator {
    private static final Class<BusinessException> EXCEPTION_TYPE = BusinessException.class;
    private static MessageSource messageSource;

    private CheckUtils() {
    }

    public static void setMessageSource(MessageSource source) {
        messageSource = source;
    }

    private static String getMessage(String key, Object... args) {
        Locale locale = LocaleContextHolder.getLocale();
        return messageSource.getMessage(key, args, locale);
    }

    public static void throwIfNotExists(Object obj, String entityName, String fieldName, Object fieldValue) {
        String message = getMessage("error.not.exists", fieldName, fieldValue, CharSequenceUtil.replace(entityName, "DO", ""));
        throwIfNull(obj, message, EXCEPTION_TYPE);
    }

    public static void throwIfNull(Object obj, String messageKey, Object... params) {
        throwIfNull(obj, getMessage(messageKey, params), EXCEPTION_TYPE);
    }

    public static void throwIfNotNull(Object obj, String messageKey, Object... params) {
        throwIfNotNull(obj, getMessage(messageKey, params), EXCEPTION_TYPE);
    }

    public static void throwIfExists(Object obj, String entityName, String fieldName, Object fieldValue) {
        String message = getMessage("error.exists", fieldName, fieldValue, entityName);
        throwIfNotNull(obj, message, EXCEPTION_TYPE);
    }

    public static void throwIfEmpty(Object obj, String messageKey, Object... params) {
        throwIfEmpty(obj, getMessage(messageKey, params), EXCEPTION_TYPE);
    }

    public static void throwIfNotEmpty(Object obj, String messageKey, Object... params) {
        throwIfNotEmpty(obj, getMessage(messageKey, params), EXCEPTION_TYPE);
    }

    public static void throwIfBlank(CharSequence str, String messageKey, Object... params) {
        throwIfBlank(str, getMessage(messageKey, params), EXCEPTION_TYPE);
    }

    public static void throwIfNotBlank(CharSequence str, String messageKey, Object... params) {
        throwIfNotBlank(str, getMessage(messageKey, params), EXCEPTION_TYPE);
    }

    public static void throwIfEqual(Object obj1, Object obj2, String messageKey, Object... params) {
        throwIfEqual(obj1, obj2, getMessage(messageKey, params), EXCEPTION_TYPE);
    }

    public static void throwIfNotEqual(Object obj1, Object obj2, String messageKey, Object... params) {
        throwIfNotEqual(obj1, obj2, getMessage(messageKey, params), EXCEPTION_TYPE);
    }

    public static void throwIfEqualIgnoreCase(CharSequence str1, CharSequence str2, String messageKey, Object... params) {
        throwIfEqualIgnoreCase(str1, str2, getMessage(messageKey, params), EXCEPTION_TYPE);
    }

    public static void throwIfNotEqualIgnoreCase(CharSequence str1, CharSequence str2, String messageKey, Object... params) {
        throwIfNotEqualIgnoreCase(str1, str2, getMessage(messageKey, params), EXCEPTION_TYPE);
    }

    public static void throwIf(boolean condition, String messageKey, Object... params) {
        throwIf(condition, getMessage(messageKey, params), EXCEPTION_TYPE);
    }

    public static void throwIf(BooleanSupplier conditionSupplier, String messageKey, Object... params) {
        throwIf(conditionSupplier, getMessage(messageKey, params), EXCEPTION_TYPE);
    }
}