package com.xs.core.controller.user;

import cn.dev33.satoken.annotation.SaIgnore;
import cn.dev33.satoken.stp.SaTokenInfo;
import com.xs.core.common.annotation.OperationLog;
import com.xs.core.model.ResponseResult;
import com.xs.core.model.user.req.AppLoginReq;
import com.xs.core.service.user.AppLoginService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Locale;

@Slf4j
@RestController
@RequestMapping("/login")
@Tag(name = "登录", description = "登录")
public class LoginController {
    @Autowired
    private AppLoginService loginService;

    @Autowired
    private MessageSource messageSource;

    @PostMapping("/coinLogin")
    @Operation(summary = "tg账号登录", description = "根据tg用户id进行登录")
    @OperationLog(description = "tg账号登录", operationType = "登录", module = "登录")
    public ResponseResult<SaTokenInfo> coinLogin(@Validated @RequestBody AppLoginReq req) {
        SaTokenInfo saTokenInfo = loginService.tgLogin(req);
        return ResponseResult.success(saTokenInfo);
    }

    @PostMapping("/testI18n")
    @Operation(summary = "测试国际化", description = "测试国际化")
    @SaIgnore
    public ResponseResult<?> testI18n() {
        Locale locale = LocaleContextHolder.getLocale();
        String message = messageSource.getMessage("error.exists", null, locale);
        return ResponseResult.success(message, "error.exists", null);
    }
}
