package com.xs.core.mapper.user;

import com.xs.core.model.team.resp.TeamUserResp;
import com.xs.core.model.user.entity.User;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.xs.core.model.user.resp.TeamCompetitionResp;
import com.xs.core.model.user.resp.UserCompetitionResp;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * tg用户 Mapper 接口
 * </p>
 *
 * @author xudm
 * @since 2024-12-22
 */
public interface UserMapper extends BaseMapper<User> {
    /**
     * 查询我的团队列表
     *
     * @param userId   用户id
     * @param teamType 团队类型
     * @return
     */
    List<TeamUserResp> getMyTeamUserList(@Param("userId") Long userId, @Param("teamType") String teamType);

    /**
     * 统计我的团队人数
     *
     * @param userId   用户id
     * @param teamType 团队类型
     * @return
     */
    int countMyTeamUserList(@Param("userId") Long userId, @Param("teamType") String teamType);

    /**
     * 金币排行榜
     *
     * @return 用户金币排行信息
     */
    List<UserCompetitionResp> individualGoldCoinCompetitionRanking(@Param("limit") int limit);

    /**
     * 金币排行榜
     *
     * @return 用户金币排行个人信息
     */
    UserCompetitionResp getIndividualGoldCoinCompetitionInfo(@Param("userId") Long userId);

    /**
     * 查询我的邀请人
     *
     * @param userId
     * @return
     */
    User getInviterByUserId(Long userId);

    /**
     * 团队邀请排行榜
     *
     * @param limit   限制
     * @param invitee 最少邀请人数
     * @return
     */
    List<TeamCompetitionResp> teamInviteCompetitionRanking(@Param("limit") int limit, @Param("invitee") int invitee);
}
