package com.xs.core.model.team.resp;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjUtil;
import com.xs.core.model.team.entity.TeamInviteConfig;
import com.xs.core.model.team.entity.TeamShareRewardRecord;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;

@Data
@Schema(name = "TeamsInfoResp", description = "团队信息")
public class TeamsInfoResp implements Serializable {
    @Schema(description = "总人数")
    private Long total = 0L;
    @Schema(description = "可领取奖励人数")
    private Long rewardsTotal = 0L;
    @Schema(description = "团队A人数")
    private Long teamACount = 0L;
    @Schema(description = "团队B人数")
    private Long teamBCount = 0L;
    @Schema(description = "团队A金币抽成占比")
    private String proportionA;
    @Schema(description = "团队B金币抽成占比")
    private String proportionB;

    @Schema(description = "团队A可领取奖励人数")
    private Long claimableRewardsA = 0L;
    @Schema(description = "团队B可领取奖励人数")
    private Long claimableRewardsB = 0L;

    @Schema(description = "团队A可领取奖励金额")
    private BigDecimal claimableAmountA = BigDecimal.ZERO;

    @Schema(description = "团队B可领取奖励金额")
    private BigDecimal claimableAmountB = BigDecimal.ZERO;

    public TeamsInfoResp(List<TeamInviteRewardsResp> inviteRewardsRespList, List<TeamUserResp> teamUserList, List<TeamShareRewardRecord> shareRewardRecordsA, List<TeamShareRewardRecord> shareRewardRecordsB, TeamInviteConfig config) {
        if (ObjUtil.isNotNull(config)) {
            this.proportionA = config.getTeamAScale();
            this.proportionB = config.getTeamBScale();
        }
        if (CollUtil.isNotEmpty(teamUserList)) {
            long countA = teamUserList.stream().filter(t -> "A".equals(t.getTeamType())).count();
            long countB = teamUserList.stream().filter(t -> "B".equals(t.getTeamType())).count();
            this.total = countA;
            this.teamACount = countA;
            this.teamBCount = countB;
        }
        if (CollUtil.isNotEmpty(inviteRewardsRespList)) {
            this.rewardsTotal = (long) inviteRewardsRespList.size();
        }
        if (CollUtil.isNotEmpty(shareRewardRecordsA)) {
            this.claimableRewardsA = (long) shareRewardRecordsA.size();
            this.claimableAmountA = shareRewardRecordsA.stream().map(TeamShareRewardRecord::getRewardAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
        }
        if (CollUtil.isNotEmpty(shareRewardRecordsB)) {
            this.claimableRewardsB = (long) shareRewardRecordsB.size();
            this.claimableAmountB = shareRewardRecordsB.stream().map(TeamShareRewardRecord::getRewardAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
        }
    }
}
