package com.xs.core.service.Impl.log;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.exceptions.ExceptionUtil;
import com.alibaba.fastjson2.JSON;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.xs.core.model.log.entity.SysAppLog;
import com.xs.core.mapper.log.SysAppLogMapper;
import com.xs.core.service.log.ISysAppLogService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.time.LocalDateTime;
import java.util.concurrent.CompletableFuture;

/**
 * <p>
 * app端系统操作日志表 服务实现类
 * </p>
 *
 * @author xudm
 * @since 2024-12-24
 */
@Service
@Slf4j
public class SysAppLogServiceImpl extends ServiceImpl<SysAppLogMapper, SysAppLog> implements ISysAppLogService {

    @Value("${interfaceLog.openInterfaceLog}")
    private boolean openInterfaceLog;

    @Override
    public void saveAppLog(String operation, String operationType, String module, String method, Object params, Object result, long time, Exception e) {
        if (!openInterfaceLog) {
            return;
        }
        SysAppLog sysLog = new SysAppLog();
        // 设置基本信息
        sysLog.setOperation(operation);
        sysLog.setOperationType(operationType);
        sysLog.setModule(module);
        sysLog.setMethod(method);
        sysLog.setCreateTime(LocalDateTime.now());
        sysLog.setExecutionTime(time);

        // 获取请求信息
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes != null) {
            HttpServletRequest request = attributes.getRequest();
            sysLog.setIp(getIpAddress(request));
            sysLog.setRequestUrl(request.getRequestURI());
            sysLog.setUserAgent(request.getHeader("User-Agent"));
        }

        // 设置参数
        if (params != null) {
            sysLog.setParams(JSON.toJSONString(params));
        }

        // 设置结果
        sysLog.setResult(JSON.toJSONString(result));
        // 设置异常信息
        // 设置操作结果
        if (e != null) {
            sysLog.setStatus(0);
            sysLog.setErrorMsg(ExceptionUtil.stacktraceToString(e));
        }

        // 设置用户信息
        if (StpUtil.isLogin()) {
            sysLog.setUserId(StpUtil.getLoginIdAsString());
        }

        // 异步保存日志
        CompletableFuture.runAsync(() -> {
            try {
                log.info("开始记录操作日志");
                save(sysLog);
            } catch (Exception ex) {
                // 处理保存日志失败的情况
                log.error("异步保存日志失败", ex);
            }
        });
    }

    // 获取IP地址
    private String getIpAddress(HttpServletRequest request) {
        String ip = request.getHeader("x-forwarded-for");
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        return ip;
    }
}
