package com.xs.core.service.Impl.user;

import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.xs.core.common.content.UserContextHolder;
import com.xs.core.common.validation.CheckUtils;
import com.xs.core.model.team.resp.TeamUserResp;
import com.xs.core.model.user.entity.User;
import com.xs.core.mapper.user.UserMapper;
import com.xs.core.model.user.req.AppLoginReq;
import com.xs.core.model.wallet.entity.UserWallet;
import com.xs.core.service.team.TeamShareService;
import com.xs.core.service.user.IUserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xs.core.service.wallet.IUserWalletService;
import com.xs.core.utils.InviteCodeGenerator;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.List;
import java.util.concurrent.CompletableFuture;

/**
 * <p>
 * tg用户 服务实现类
 * </p>
 *
 * @author xudm
 * @since 2024-12-22
 */
@Service

public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    @Autowired
    private IUserWalletService walletService;

    @Autowired
    @Lazy
    private TeamShareService teamShareService;

    @Override
    public User getInviterByUserId(Long userId) {
        return getBaseMapper().getInviterByUserId(userId);
    }

    @Override
    public List<TeamUserResp> getMyTeamUserList(Long userId, String teamType) {
        return getBaseMapper().getMyTeamUserList(userId, teamType);
    }

    @Override
    public String getInviteCode() {
        Long userId = UserContextHolder.getContext().getId();
        User user = getById(userId);
        CheckUtils.throwIfNull(user, "response.unauthorized");
        String inviteCode = user.getInviteCode();
        if (StrUtil.isBlank(inviteCode)) {
            inviteCode = InviteCodeGenerator.generateInviteCode();
            user.setInviteCode(inviteCode);
            updateById(user);
            return inviteCode;
        }
        return inviteCode;
    }

    @Override
    public Long queryInviterCount(Long id) {
        LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(User::getReferrerId, id);
        return count(queryWrapper);
    }

    @Override
    public boolean checkUserExist(Long aLong) {
        LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(User::getId, aLong);
        User user = getOne(queryWrapper);
        return ObjUtil.isNotNull(user);
    }

    @Override
    @Transactional
    public User registerToSys(AppLoginReq loginReq) {
        User user = User.builder()
                .id(IdWorker.getId())
                .tgAccount(loginReq.getUsername())
                .tgId(loginReq.getId())
                .avatar(StrUtil.isNotBlank(loginReq.getPhotoUrl()) ? URLDecoder.decode(loginReq.getPhotoUrl(), StandardCharsets.UTF_8) : "")
                .firstName(loginReq.getFirstName())
                .lastName(loginReq.getLastName())
                //注册时指定用户的唯一邀请码
                .inviteCode(InviteCodeGenerator.generateInviteCode())
                .languageCode(loginReq.getLanguageCode())
                .createdTime(LocalDateTime.now())
                .updatedTime(LocalDateTime.now())
                .build();
        if (StrUtil.isNotBlank(loginReq.getAge_limit())) {
            user.setAgeLimit(Integer.parseInt(loginReq.getAge_limit()));
        }
        User inviteUser;
        if (StrUtil.isNotBlank(loginReq.getShare_code())) {
            //如邀请码不为空 则写入邀请人信息
            LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(User::getInviteCode, loginReq.getShare_code());
            inviteUser = getOne(queryWrapper);
            if (ObjUtil.isNotNull(inviteUser)) {
                user.setReferrerId(String.valueOf(inviteUser.getId()));
                user.setPassengerFlowWay(2);
                user.setId(IdWorker.getId());
            }
        } else {
            inviteUser = null;
        }
        //初始化钱包信息
        UserWallet userWallet = new UserWallet();
        userWallet.setUserId(user.getId());
        userWallet.setUpdatedTime(LocalDateTime.now());
        save(user);
        walletService.save(userWallet);
        CompletableFuture.runAsync(() -> {
            //注册成功后发放邀请奖励
            if (ObjUtil.isNotNull(inviteUser)) {
                teamShareService.teamInviteGoldCoinRewardGeneral(inviteUser.getId(), user.getId());
            }
        });
        return user;
    }
}
