package com.xs.core.service.team;

import com.xs.core.model.coin.msg.TeamShareMessage;
import com.xs.core.model.team.req.TeamShareReq;
import com.xs.core.model.team.resp.*;

import java.util.List;

/**
 * 团队相关接口
 */
public interface TeamShareService {

    /**
     * 团队产出金币结算
     *
     * @param teamShareMessage 团队产出消息体
     */
    void teamShareGoldCoinSettlement(TeamShareMessage teamShareMessage);

    /**
     * 团队邀请奖励发放
     *
     * @param inviterId 邀请人id
     * @param inviteeId 被邀请人id
     */
    void teamInviteGoldCoinRewardGeneral(Long inviterId, Long inviteeId);

    /**
     * 获取我的团队信息
     *
     * @param id 用户id
     */
    TeamsInfoResp getMyTeamsInfo(Long id);

    /**
     * 获取团队产出用户信息
     *
     * @param id  用户id
     * @param req 请求体
     */
    TeamShareInfoResp getTeamShareUserInfoByType(Long id, TeamShareReq req);

    /**
     * 收取团队产出奖励
     *
     * @param id  用户id
     * @param req 请求体
     */
    void claimTeamShareReward(Long id, TeamShareReq req);

    /**
     * 获取团队邀请奖励列表 固定奖励 邀请1人奖励1000金币
     *
     * @param id 用户id
     */
    List<TeamInviteRewardsResp> getTeamInviteRewardsList(Long id);

    /**
     * 收取团队邀请奖励 固定奖励 邀请1人奖励1000金币
     *
     * @param id 用户id
     */
    void claimTeamInviteReward(Long id);

    /**
     * 获取团队邀请任务可领取奖励
     *
     * @param id 用户id
     */
    TeamInviteAvailableRewardsResp getTeamInviteAvailableRewardsRules(Long id);

    /**
     * 领取团队邀请任务奖励
     *
     * @param id 用户id
     */
    void claimTeamInviteTaskReward(Long id, String ruleId);
}
